#ifndef __TAPTEMPOAVERAGE_HPP__             /*-*- indent-tabs-mode:nil;-*-*/
#define __TAPTEMPOAVERAGE_HPP__
/***************************************************************************
 *                                                                         *
 *   InConcert - Tap Tempo Synchronization for Live Perforances            *
 *                                                                         *
 *   Copyright (C) 2008 by Gabriel M. Beddingfield                         *
 *   gabriel@teuton.org                                                    *
 *                                                                         *
 *   "For of him [God], and through him, and unto him, are all things.     *
 *   To him be the glory for ever. Amen." (Romans 11:36)                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; version 2 of the License, or any later  *
 *   version.                                                              *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

/* TapTempoAverage.hpp
   Gabriel M. Beddingfield
   2008-08-16

 */

#include "TapTempoBaseImpl.hpp"

namespace InConcert
{
    // TapTempoAverage
    // Tn+1 = Tn + (Tn-T0)/n
    template<typename ScalarT, typename TempoT = double>
    class TapTempoAverage : public TapTempoBaseImpl<ScalarT, TempoT>
    {
        ScalarT m_first;
        unsigned m_taps_received;
        ScalarT m_last_tap;
        QMutex m_mutex;
        typedef TapTempoBaseImpl<ScalarT, TempoT> base_class;

    public:
        TapTempoAverage() :
            base_class("TapTempoAverage"),
            m_first(0),
            m_taps_received(0),
            m_last_tap(0)
            { }
        virtual void invalidate(void) {
            QMutexLocker mx(&m_mutex);
            m_taps_received = 0;
            m_first = 0;
            base_class::invalidate();
        }

        virtual ScalarT tap(ScalarT tap) {
            if (this->reset() || (m_taps_received >= this->getTapsRequired())) {
                invalidate();
            } else if (this->getTimeout()
                       && (tap - m_last_tap) >= this->getTimeout()) {
                invalidate();
            }

            QMutexLocker mx(&m_mutex);
            assert(tap > m_first);

            ScalarT rv = 0;
            ++m_taps_received;
            m_last_tap = tap;
            if(m_taps_received == 1) {
                m_first = tap;
            }
            if(m_taps_received == this->getTapsRequired()) {
                ScalarT intervals = m_taps_received - 1;
                setTempoByInterval( (tap-m_first)/intervals );
                rv = tap + this->getTempoInterval();
                this->queueReset();
            }
            return rv;
        }

    }; // class TapTempoAverage

} // namespace InConcert   
#endif // __TAPTEMPOAVERAGE_HPP__
